#include "lcd.h"
#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>

#define KEYPAD_IN   PINB
#define KEYPAD_OUT  PORTB
#define KEYPAD_DIR  DDRB
#define COL1        PB4
#define COL2        PB3
#define COL3        PB2
#define ROW1		PB1
#define ROW2		PB0
#define RED			~_BV(PB5)
#define YELLOW		~_BV(PB6)
#define GREEN		~_BV(PB7)

void delay1(void)
{
	unsigned char i;
	for (i=0;i<255;i++)
		asm("nop");
}

void delay2(void)
{
	unsigned int i;
	for (i=0;i<4096;i++)
		asm("nop");
}

unsigned char keyscan(void)
{

unsigned char row, col, key;

// find row pressed
KEYPAD_DIR = 0xFC;
KEYPAD_OUT = (KEYPAD_OUT & 0xE0) | 0x03;
delay1();
row = KEYPAD_IN;			// read 4 row pins, active low
row = (~row) & 0x03;

// change port I/0 to find column pressed
KEYPAD_DIR = 0xE3;
KEYPAD_OUT = (KEYPAD_OUT & 0xE0) | 0x1C;
delay1();					// wait until pin has changed from output to input
col = KEYPAD_IN;			// read 4 col pins, active low
col = (~col) & 0x1C;
key = 0;
if ( (row != 0) && (col != 0) ) {
	if ( row & _BV(ROW2) )
		key = 1; // 0
	else if ( row & _BV(ROW1) )
		key = 4; // 3
	if ( col & _BV(COL3) )
		key = key + 0;
	else if (col & _BV(COL2) )
		key = key + 1;
	else if (col & _BV(COL1) )
		key = key + 2;
	}
return key; 
}

int main(void)
{
unsigned char keycode;

/* register settings */
DDRB=0x0F;							// PortB: 0..3 out, 4..7 in
DDRD=0x7F;							// PortD: all out, later controlled by library
PORTB=0xFF;							// activate PullUp's, switch off LEDs
PORTD=0x00;							// all off

lcd_init(LCD_DISP_ON);
PORTB |= 0xE0;		            	// switch off all LEDs
lcd_clrscr();
lcd_puts_P("doorsystem restarted");

/* german "Umlaute":     0xF5    0xEF    0xE1    0xE2 */

while(1)
	{
	keycode = keyscan();
	if ( keycode > 0 )
		{
		delay1();
		switch (keycode)
			{
			case 1:
			PORTB |= 0xE0;			// switch off all LEDs
			PORTB &= YELLOW;		// turn on required LED
			lcd_clrscr();
			lcd_puts_P("# niemand zu Hause #");
			break;
			
			case 2:
			PORTB |= 0xE0;			// switch off all LEDs
			PORTB &= YELLOW;		// turn on required LED
			lcd_clrscr();
			lcd_puts_P("> Ich habe Besuch. <");
			break;
			
			case 3:
			PORTB |= 0xE0;			// switch off all LEDs
			PORTB &= GREEN;			// turn on required LED
			lcd_clrscr();
			// lcd_puts_P("hast Du Bier dabei?");
			lcd_puts_P("Bierchen? Komm rein!");
			break;
			
			case 4:
			PORTB |= 0xE0;			// switch off all LEDs
			PORTB &= GREEN;			// turn on required LED
			lcd_clrscr();
			lcd_puts_P("Herzlich Willkommen!");
			break;

			case 5:
			PORTB |= 0xE0;			// switch off all LEDs
			PORTB &= RED;			// turn on required LED
			lcd_clrscr();
			lcd_puts(" Bitte nicht st");
			lcd_putc(0xEF); // 
			lcd_puts("ren");
			break;
			
			case 6:
			PORTB |= 0xE0;			// switch off all LEDs
			PORTB &= RED;			// turn on required LED
			lcd_clrscr();
			lcd_puts_P("! Zutritt verboten !");
			break;
			}
		}
	delay2();
	}
}



